@@ -38,6 +38,8 @@ amdgpu_get_gbm_format(int depth, int bitsPerPixel)
 	case 8:
 		return GBM_FORMAT_R8;
 #endif
+	case 15:
+		return GBM_FORMAT_ARGB1555;
 	case 16:
 		return GBM_FORMAT_RGB565;
 	case 32:
@@ -77,7 +79,7 @@ struct amdgpu_buffer *amdgpu_alloc_pixmap_bo(ScrnInfoPtr pScrn, int width,
 		}
 		pixmap_buffer->ref_count = 1;
 
-		if ( bitsPerPixel == pScrn->bitsPerPixel)
+		if (usage_hint & AMDGPU_CREATE_PIXMAP_SCANOUT)
 			bo_use |= GBM_BO_USE_SCANOUT;
 
 #ifdef HAVE_GBM_BO_USE_LINEAR
@@ -148,6 +150,8 @@ Bool amdgpu_bo_get_handle(struct amdgpu_buffer *bo, uint32_t *handle)
 				handle) == 0;
 }
 
+#ifdef USE_GLAMOR
+
 static void amdgpu_pixmap_do_get_tiling_info(PixmapPtr pixmap)
 {
 	struct amdgpu_pixmap *priv = amdgpu_get_pixmap_private(pixmap);
@@ -164,6 +168,8 @@ static void amdgpu_pixmap_do_get_tiling_info(PixmapPtr pixmap)
 		priv->tiling_info = gem_metadata.data.tiling_info;
 }
 
+#endif
+
 uint64_t amdgpu_pixmap_get_tiling_info(PixmapPtr pixmap)
 {
 	struct amdgpu_pixmap *priv = amdgpu_get_pixmap_private(pixmap);
@@ -179,13 +185,15 @@ uint64_t amdgpu_pixmap_get_tiling_info(PixmapPtr pixmap)
 
 Bool amdgpu_pixmap_get_handle(PixmapPtr pixmap, uint32_t *handle)
 {
-#ifdef USE_GLAMOR
 	ScreenPtr screen = pixmap->drawable.pScreen;
 	ScrnInfoPtr scrn = xf86ScreenToScrn(screen);
 	AMDGPUInfoPtr info = AMDGPUPTR(scrn);
-#endif
-	struct amdgpu_pixmap *priv = amdgpu_get_pixmap_private(pixmap);
+	struct amdgpu_pixmap *priv;
 
+	if (info->shadow_fb)
+		return FALSE;
+
+	priv = amdgpu_get_pixmap_private(pixmap);
 	if (!priv) {
 		priv = calloc(1, sizeof(*priv));
 		amdgpu_set_pixmap_private(pixmap, priv);
@@ -207,16 +215,17 @@ Bool amdgpu_pixmap_get_handle(PixmapPtr pixmap, uint32_t *handle)
 
 		r = drmPrimeFDToHandle(pAMDGPUEnt->fd, fd, &priv->handle);
 		close(fd);
-		if (r == 0)
-			goto get_tiling_info;
+		if (r)
+			return FALSE;
+
+		amdgpu_pixmap_do_get_tiling_info(pixmap);
+		goto success;
 	}
 #endif
 
 	if (!priv->bo || !amdgpu_bo_get_handle(priv->bo, &priv->handle))
 		return FALSE;
 
- get_tiling_info:
-	amdgpu_pixmap_do_get_tiling_info(pixmap);
  success:
 	priv->handle_valid = TRUE;
 	*handle = priv->handle;
