@@ -201,14 +201,25 @@ amdgpu_glamor_create_pixmap(ScreenPtr screen, int w, int h, int depth,
 			    unsigned usage)
 {
 	ScrnInfoPtr scrn = xf86ScreenToScrn(screen);
+	PixmapFormatPtr format = xf86GetPixFormat(scrn, depth);
 	AMDGPUInfoPtr info = AMDGPUPTR(scrn);
 	struct amdgpu_pixmap *priv;
 	PixmapPtr pixmap, new_pixmap = NULL;
 
-	if (!xf86GetPixFormat(scrn, depth))
+	if (!format)
 		return NULL;
 
-	if (!AMDGPU_CREATE_PIXMAP_SHARED(usage)) {
+	if (usage != CREATE_PIXMAP_USAGE_BACKING_PIXMAP &&
+	    usage != CREATE_PIXMAP_USAGE_SHARED &&
+	    !info->shadow_primary &&
+	    w >= scrn->virtualX &&
+	    w <= scrn->displayWidth &&
+	    h == scrn->virtualY &&
+	    format->bitsPerPixel == scrn->bitsPerPixel)
+		usage |= AMDGPU_CREATE_PIXMAP_SCANOUT;
+
+	if (!(usage & AMDGPU_CREATE_PIXMAP_SCANOUT) &&
+	    !AMDGPU_CREATE_PIXMAP_SHARED(usage)) {
 		if (info->shadow_primary) {
 			if (usage != CREATE_PIXMAP_USAGE_BACKING_PIXMAP)
 				return fbCreatePixmap(screen, w, h, depth, usage);
@@ -342,7 +353,7 @@ amdgpu_glamor_set_pixmap_bo(DrawablePtr drawable, PixmapPtr pixmap)
 
 
 static Bool
-amdgpu_glamor_share_pixmap_backing(PixmapPtr pixmap, ScreenPtr slave,
+amdgpu_glamor_share_pixmap_backing(PixmapPtr pixmap, ScreenPtr secondary,
 				   void **handle_p)
 {
 	ScreenPtr screen = pixmap->drawable.pScreen;
